#include <net/if.h>		/* struct ifreq */
#include <sys/ioctl.h>		/* ioctl */

#include "boa.h"


/******************************************************************	
    args:
		ip	-- the ip you want to know which interface come from
	return:
		"br0"
		"eth1"
*******************************************************************/ 
#define LAN_ID 0
#define WAN_ID 1
#define PPP_ID 2
char *from_interface(uint32_t ip)
{
	int fd;

	static char *from[] = { "br0", "eth1", "ppp0" };
	uint32_t addr, mask;
	struct sockaddr_in *dev_ip;
	struct ifreq ifr;

	if ((fd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) >= 0){
		ifr.ifr_addr.sa_family = AF_INET;
		strcpy(ifr.ifr_name, from[LAN_ID]);

		if (ioctl(fd, SIOCGIFADDR, &ifr) == 0){
			dev_ip = (struct sockaddr_in *) &ifr.ifr_addr;
			addr = dev_ip->sin_addr.s_addr;
		} else {
			close(fd);
			return NULL;
		}

		if (ioctl(fd, SIOCGIFNETMASK, &ifr) == 0){
			dev_ip = (struct sockaddr_in *) &ifr.ifr_addr;
			mask = dev_ip->sin_addr.s_addr;
		} else {
			close(fd);
			return NULL;
		}


		if ( (addr & mask) == (ip & mask) ){
			close(fd);
			return from[LAN_ID];
		}

		close(fd);
	}

	char wan_proto[16];
	sprintf(wan_proto, "%s", config_get("wan_proto"));
	if(strcmp(wan_proto, "pppoe") == 0
		|| strcmp(wan_proto, "pptp") == 0) {
		return from[PPP_ID];
	} else {
		return from[WAN_ID];
	}

#if 0
	if ((fd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) >= 0){
		ifr.ifr_addr.sa_family = AF_INET;
		strcpy(ifr.ifr_name, WAN);

		if (ioctl(fd, SIOCGIFADDR, &ifr) == 0){
			dev_ip = (struct sockaddr_in *) &ifr.ifr_addr;
			addr = dev_ip->sin_addr.s_addr;
		} else {
			close(fd);
			return NULL;
		}

		if (ioctl(fd, SIOCGIFNETMASK, &ifr) == 0){
			dev_ip = (struct sockaddr_in *) &ifr.ifr_addr;
			mask = dev_ip->sin_addr.s_addr;
		} else {
			close(fd);
			return NULL;
		}


		if ( (addr & mask) == (ip & mask) ){
			close(fd);
			return from[WAN_ID];
		}

		close(fd);
	}

	return NULL;
#endif
}


/**************************************************************************
    args:
		interface		-- interface to use
		addr			-- get interface's ip address
		arp				-- get interface's mac address
	return:
		0		-- success
		-1		-- fail
****************************************************************************/
int read_interface(char *interface, uint32_t *addr, uint32_t *mask, uint8_t *arp)
{
	int fd;
	struct ifreq ifr;
	struct sockaddr_in *ourip;

	memset(&ifr, 0, sizeof(struct ifreq));
	if ((fd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0) {
		print2screen("%s, %d: socket error\n", __FUNCTION__, __LINE__);
		return -1;
	}

	ifr.ifr_addr.sa_family = AF_INET;
	strcpy(ifr.ifr_name, interface);

	if (addr) {
		if (ioctl(fd, SIOCGIFADDR, &ifr) == 0) {
			ourip = (struct sockaddr_in *)&ifr.ifr_addr;
			*addr = ourip->sin_addr.s_addr;
		} else {
		    close(fd);
			return -1;
		}
	}
	if (mask) {
		if (ioctl(fd, SIOCGIFNETMASK, &ifr) == 0) {
			ourip = (struct sockaddr_in *)&ifr.ifr_addr;
			*mask = ourip->sin_addr.s_addr;
		} else {
		    close(fd);
			return -1;
		}
	}

	if (arp) {
		if (ioctl(fd, SIOCGIFHWADDR, &ifr) == 0) {
			memcpy(arp, ifr.ifr_hwaddr.sa_data, 6);
		} else {
		    close(fd);
			return -1;
		}
	}

	close(fd);

	return 0;
}


/**********************************************************************
*   args: 
*	yiaddr	-- what IP to ping
*	yimac	-- what mac we want to get
*	ip		-- our ip
*	mac		-- our arp address

*  return:
*	1		-- addr free
*	0		-- addr used
*	-1		-- error
************************************************************************/
int arpping(uint32_t yiaddr, uint8_t *ymac, uint32_t ip, uint8_t *mac)
{
	char *interface;
	int timeout = 2;
	int optval = 1;
	int s;		/* socket */
	struct arpMsg arp;
	struct sockaddr addr;	/* for interface name */
	struct timeval tm;
	long prevTime;
	fd_set fdset;
	int rv = 1;			/* return value */

	uint32_t lan_ip;
	uint8_t  lan_mac[6];

	if ((interface = from_interface(yiaddr)) == NULL) {
		return -1;
	}

#if 1
	if(strcmp(interface, "ppp0") == 0) {
		memset(ymac, 0xff, 6);
		return 1;
	}
#endif
	
	if (read_interface(interface, &lan_ip, NULL, lan_mac) == 0) {
		if (ip == 0)
			ip = lan_ip;
		if (mac == NULL)
			mac = lan_mac;
	}

	if ((s = socket(PF_PACKET, SOCK_PACKET, htons(ETH_P_ARP))) == -1) {
		print2screen("%s, %d: socket error\n", __FUNCTION__, __LINE__);
		return -1;
	}

	if (setsockopt(s, SOL_SOCKET, SO_BROADCAST, &optval, sizeof(optval)) == -1) {
		print2screen("%s, %d: setsocket error\n", __FUNCTION__, __LINE__);
		close(s);
		return -1;
	}

	/* send arp request */
	memset(&arp, 0, sizeof(arp));
	memset(arp.h_dest, 0xff, 6);	/* MAC DA, broadcast */
	memcpy(arp.h_source, mac, 6);	/* MAC SA */
	arp.h_proto = htons(ETH_P_ARP);	/* protocol type (Ethernet) */

	arp.htype = htons(ARPHRD_ETHER);	/* hardware type */
	arp.ptype = htons(ETH_P_IP);	/* protocol type (ARP message) */
	arp.hlen = 6;					/* hardware address length */
	arp.plen = 4;					/* protocol address length */
	arp.operation = htons(ARPOP_REQUEST);	/* ARP op code */
	memcpy(arp.sInaddr, &ip, sizeof(ip));	/* source IP address */
	memcpy(arp.sHaddr, mac, 6);				/* source hardware address */
	memcpy(arp.tInaddr, &yiaddr, sizeof(yiaddr));	/* target IP address */

	memset(&addr, 0, sizeof(addr));
	strcpy(addr.sa_data, interface);
	if (sendto(s, &arp, sizeof(arp), 0, &addr, sizeof(addr)) < 0) {
		print2screen("%s, %d: sendto error\n", __FUNCTION__, __LINE__);
		close(s);
		return -1;
	}

	/* wait arp reply, and check it */
	tm.tv_usec = 0;
	prevTime = uptime();
	while (timeout > 0) {
		FD_ZERO(&fdset);
		FD_SET(s, &fdset);
		tm.tv_sec = timeout;
		if (select(s+1, &fdset, NULL, NULL, &tm) < 0) {
			print2screen("%s, %d: Error on ARPING request\n", __FUNCTION__, __LINE__);
			if (errno != EINTR)
				continue;
		} else if (FD_ISSET(s, &fdset)) {
			if (recv(s, &arp, sizeof(arp), 0) < 0)
				continue;
			if (arp.operation == htons(ARPOP_REPLY)
				&& memcmp(arp.tHaddr, mac, 6) == 0
				&& *((uint32_t *)arp.sInaddr) == yiaddr) {
				/* get what we want*/
				memcpy(ymac, arp.sHaddr, 6);
		//		print2screen("get mac %.02x:%.02x:%.02x:%.02x:%.02x:%.02x\n",
		//						ymac[0], ymac[1], ymac[2], ymac[3],
		//						ymac[4], ymac[5]);
				rv = 0;
				break;
			}
		}
		timeout -= uptime() - prevTime;
		prevTime = uptime();
	}
	close(s);

	if(rv == 1) {
		memset(ymac, 0xff, 6);
	}

	return rv;
}


/**************************************************************************
*   return:
*		0		-- legal user
*		1		-- illegal user
***************************************************************************/
#define LEGAL_USER 0
#define ILLEGAL_USER 1
int user_is(request *req)
{
	uint32_t yiaddr;
	uint8_t ymac[6];
	
	if ( auth_info.ip[0] && strcmp(auth_info.ip, req->remote_ip_addr) != 0) {
		inet_aton(req->remote_ip_addr, (struct in_addr *)&yiaddr);
		if (arpping(yiaddr, ymac, 0, NULL) >= 0) {
			if (memcmp(auth_info.mac, ymac, 6) != 0) {
				return ILLEGAL_USER;
			}
		}
	}

	return LEGAL_USER;
}


/***************************************************************************  
*   return:
*       length of decoded string
*
****************************************************************************/
int base64decode(void *dst,char *src,int maxlen)
{
	int bitval,bits;
	int val;
	int len,x,y;
	
	len = strlen(src);
	bitval=0;
	bits=0;
	y=0;

	for(x=0;x<len;x++)	{
		if ((src[x]>='A')&&(src[x]<='Z')) val=src[x]-'A'; else
			if ((src[x]>='a')&&(src[x]<='z')) val=src[x]-'a'+26; else
				if ((src[x]>='0')&&(src[x]<='9')) val=src[x]-'0'+52; else
					if (src[x]=='+') val=62; else
						if (src[x]=='-') val=63; else
							val=-1;
		if (val>=0)			{
			bitval=bitval<<6;
			bitval+=val;
			bits+=6;
			while (bits>=8)				{
				if (y<maxlen)
					((char *)dst)[y++]=(bitval>>(bits-8))&0xFF;
				bits-=8;
				bitval &= (1<<bits)-1;
			}
		}
	}
	if (y<maxlen)
		((char *)dst)[y++]=0;
	return y;
}


/******************************************************************************
*   return:
*   	0			-- match
*   	1			-- unmatch
*******************************************************************************/
#define MATCH 0
#define UNMATCH 1
int check_passwd(request *req)
{
	char password[64], user[64], admin_password[128], userpass[128];
	memset(password, 0, sizeof(password));
	memset(user, 0, sizeof(user));
	memset(admin_password, 0, sizeof(admin_password));
	memset(userpass, 0, sizeof(userpass));

	memset(auth_info.username, 0, sizeof(auth_info.username));
	memset(auth_info.password, 0, sizeof(auth_info.password));

	sprintf(auth_info.username, "%s", config_get("http_username"));
	sprintf(auth_info.password, "%s", config_get("http_passwd"));
	
	/* check username & password */
	base64decode(userpass, req->authorization+6, 0x100);
	int len = 0;
	len += sprintf(admin_password + len, "%s", auth_info.username);
	len += sprintf(admin_password + len, "%s", ":");
	len += sprintf(admin_password + len, "%s", auth_info.password);
	if (strcmp(userpass, admin_password) == 0) {
		return MATCH;
	}

	return UNMATCH;
}


/******************************************************************************
*   return
*	0	-- don't need authentication
*	1	-- need authentication
*******************************************************************************/
int need_auth()
{
	memset(auth_info.password, 0, sizeof(auth_info.password));

	sprintf(auth_info.password, "%s", config_get("http_passwd"));

	if (strcmp(auth_info.password, "") == 0)
		return 0;	/* don't need authentication */
	else
		return 1;	/* need authentication */
}

/*******************************************************************************
 * arg:
 * 	ip_string	-- IP you want to save to file
 *****************************************************************************/
inline void set_remote_ip(char *ip_string)
{
	config_set("remote_ip", ip_string);
}

/****************************************************************************
 * arg:
 * 	mac		-- 6 octet value
 ***************************************************************************/
inline void set_remote_mac(unsigned char *mac)
{
	char mac_string[20];
	sprintf(mac_string, "%02x:%02x:%02x:%02x:%02x:%02x",
			mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
	config_set("wan_remote_mac", mac_string);
}


/***************************************************************************** 
*   return
*	0	-- success
*	1	-- fail
******************************************************************************/

int do_auth(request *req)
{
	char password[64], user[64], admin_password[128];
	char *interface;
	uint32_t yiaddr;
	uint8_t ymac[6];

	memset(password, 0, sizeof(password));
	memset(user, 0, sizeof(user));
	memset(admin_password, 0, sizeof(admin_password));

#if 0
	print2screen("\n-------------------------------------------------\n");

	print2screen("\nphase = %d, ip = %s, lat = %d, mac[1] = %d, password = %s, username = %s\n",
				auth_info.auth_phase, auth_info.ip, auth_info.last_access_time, 
				auth_info.mac[1], auth_info.password, auth_info.username);
#endif

	
	if (memcmp(req->pathname,"/www/cgi-bin/currentsetting.htm", 32) == 0 || 
		strncmp(req->remote_ip_addr, "127.0.0.1", 9) == 0) {
		return 0;
	}

	if (strcmp(config_get("dns_hijack"),"1") == 0)
        {
                if (memcmp(auth_info.mac, (uint8_t *)"\x00\x00\x00\x00\x00\x00", 6) == 0){
                        inet_aton(req->remote_ip_addr, (struct in_addr *)&yiaddr);
                        if (arpping(yiaddr, ymac, 0, NULL) >= 0) {
                                //memcpy(auth_info.mac, ymac, 6);
                                /* update remote_mac */
                                set_remote_mac(ymac);
                        }
                }

                return 0;
        }
#if 1
	if (strstr(req->pathname, ".html") == NULL) {
		return 0;
	}
#else
	if (strstr(req->pathname, "languages.js")) {
		return 0;
	}
#endif
	if (memcmp(req->pathname,"/www/cgi-bin/logout.html", 25) == 0) {
		init_auth_info();
		return 0;
	}

	if ((uptime() - auth_info.last_access_time > EXPIRE_TIME) 
		&& (auth_info.auth_phase == AUTHED /* || auth_info.auth_phase == MULTI_LOGIN */)) {
		/* init all authentic information */
		init_auth_info();
	}

#if 1
	if (user_is(req) == ILLEGAL_USER) {
		/* if DUT's password is NULL, don't need authentication */
		if (need_auth() == 0) {
			goto fail_by_multilogin;
		}
		
		if ((!(req->authorization && strncasecmp(req->authorization, "Basic ", 6) == 0))
			|| check_passwd(req) == UNMATCH) 
		{
			/* password check fail */

			auth_info.illegal_phase = MULTI_LOGIN;

			send_r_unauthorized(req, auth_realm);
			if (req->authorization && strncasecmp(req->authorization, "Basic ", 6) == 0) {
				syslog(6, "[%s login failure] from source %s,", from_subnet(req), req->remote_ip_addr);
			}
			flush_all_queue(req);
			return 1;
		}
		else if (auth_info.illegal_phase == UN_AUTH)
		{
			auth_info.illegal_phase = MULTI_LOGIN;
			send_r_unauthorizedtimeout(req, auth_realm);
			flush_all_queue(req);
			return 1;
		}
		else if (memcmp(req->pathname,"/www/cgi-bin/ptimeout.html", 27) == 0)
		{
			auth_info.illegal_phase = UN_AUTH;
			send_r_forbidden(req);
			syslog(6, "[%s login failure] from source %s,", from_subnet(req), req->remote_ip_addr);
			flush_all_queue(req);
			return 1;
		}

fail_by_multilogin:	
	
		auth_info.illegal_phase = UN_AUTH;
		send_r_unauthorizedlogin(req);
		syslog(6, "[%s login failure] from source %s,", from_subnet(req), req->remote_ip_addr);
		flush_all_queue(req);
		return 1;

	} else {	/* legal user */

		/* if DUT's password is NULL, don't need authentication */
		if (need_auth() == 0) {
			goto login_success;
		}
		if ((!(req->authorization && (strncasecmp(req->authorization, "Basic ", 6) == 0)))
			|| check_passwd(req) == UNMATCH ) 
		{
			/* password check fail */

			auth_info.auth_phase = AUTHING;
			send_r_unauthorized(req, auth_realm);
			if (req->authorization && strncasecmp(req->authorization, "Basic ", 6) == 0) {
				syslog(6, "[%s login failure] from source %s,", from_subnet(req), req->remote_ip_addr);
			}
			flush_all_queue(req);
			return 1;
		} 
		else if (auth_info.auth_phase == UN_AUTH)
		{
			auth_info.auth_phase = AUTHING;
			send_r_unauthorizedtimeout(req, auth_realm);
			flush_all_queue(req);
			return 1;
		}
		else if (memcmp(req->pathname,"/www/cgi-bin/ptimeout.html", 27) == 0)
		{
			auth_info.auth_phase = UN_AUTH;
			send_r_forbidden(req);
			syslog(6, "[%s login failure] from source %s,", from_subnet(req), req->remote_ip_addr);
			flush_all_queue(req);
			return 1;
		}

login_success:
			
		/* update auth information */

		auth_info.last_access_time = uptime();
		auth_info.auth_phase = AUTHED;
		if (!auth_info.ip[0] || strcmp(auth_info.ip, req->remote_ip_addr) != 0){
			strcpy(auth_info.ip, req->remote_ip_addr);
			/* update remote_ip file*/
			set_remote_ip(req->remote_ip_addr);		
#if 1			
			inet_aton(req->remote_ip_addr, (struct in_addr *)&yiaddr);
			if (arpping(yiaddr, ymac, 0, NULL) >= 0) {
				memcpy(auth_info.mac, ymac, 6);
				/* update remote_mac */
				set_remote_mac(ymac);
			}
#endif
			syslog(6, "[%s login] from source %s,", from_subnet(req), req->remote_ip_addr);
		}
#if 1
		if (memcmp(auth_info.mac, (uint8_t *)"\x00\x00\x00\x00\x00\x00", 6) == 0){
			inet_aton(req->remote_ip_addr, (struct in_addr *)&yiaddr);
			if (arpping(yiaddr, ymac, 0, NULL) >= 0) {
				memcpy(auth_info.mac, ymac, 6);
				/* update remote_mac */
				set_remote_mac(ymac);
			}
		}
#endif
		return 0;
	}	/* legal user end */



#endif
#if 0
	print2screen("\nphase = %d, ip = %s, lat = %d, mac[1] = %d, password = %s, username = %s\n",
				auth_info.auth_phase, auth_info.ip, auth_info.last_access_time, 
				auth_info.mac[1], auth_info.password, auth_info.username);

	print2screen("\n++++++++++++++++++++++++++++++++++++++++++++++\n");
#endif
	return 0;
}
		
